/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_MediaTrackConstraintSetBinding_h
#define mozilla_dom_MediaTrackConstraintSetBinding_h

#include "ConstraintsBinding.h"
#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct ConstrainDoubleRange;
struct ConstrainLongRange;
struct MediaTrackConstraintSetAtoms;
struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

MOZ_BEGIN_ENUM_CLASS(SupportedVideoConstraints, uint32_t)
  Other,
  FacingMode,
  Width,
  Height,
  FrameRate,
  EndGuard_
MOZ_END_ENUM_CLASS(SupportedVideoConstraints)

namespace SupportedVideoConstraintsValues {
extern const EnumEntry strings[6];
} // namespace SupportedVideoConstraintsValues


MOZ_BEGIN_ENUM_CLASS(SupportedAudioConstraints, uint32_t)
  Other,
  EndGuard_
MOZ_END_ENUM_CLASS(SupportedAudioConstraints)

namespace SupportedAudioConstraintsValues {
extern const EnumEntry strings[2];
} // namespace SupportedAudioConstraintsValues


struct MediaTrackConstraintSet : public DictionaryBase
{
  Optional<VideoFacingModeEnum> mFacingMode;
  ConstrainDoubleRange mFrameRate;
  ConstrainLongRange mHeight;
  ConstrainLongRange mWidth;

  MediaTrackConstraintSet();

  explicit inline MediaTrackConstraintSet(const MediaTrackConstraintSet& aOther)
  {
    *this = aOther;
  }

protected:
  explicit inline MediaTrackConstraintSet(int )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

private:
  static bool
  InitIds(JSContext* cx, MediaTrackConstraintSetAtoms* atomsCache);

public:
  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value");

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const MediaTrackConstraintSet& aOther);
};

namespace binding_detail {
struct FastMediaTrackConstraintSet : public MediaTrackConstraintSet
{
  inline FastMediaTrackConstraintSet()
    : MediaTrackConstraintSet(42)
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MediaTrackConstraintSetBinding_h
